about summary refs log tree commit diff
path: root/src
diff options
context:
space:
mode:
authorRich Felker <dalias@aerifal.cx>2011-02-16 09:49:54 -0500
committerRich Felker <dalias@aerifal.cx>2011-02-16 09:49:54 -0500
commit22cd9cae819a0fd432da4b1d21502a242871e5d9 (patch)
tree39aaffd5e4fb3feacb755ad4d5a02c0bb096dc28 /src
parentbad481266e8744f8c4241e577f06e8cabf6f7f89 (diff)
downloadmusl-22cd9cae819a0fd432da4b1d21502a242871e5d9.tar.gz
musl-22cd9cae819a0fd432da4b1d21502a242871e5d9.tar.xz
musl-22cd9cae819a0fd432da4b1d21502a242871e5d9.zip
patch from njk: make x86_64 __uniclone branchless.
Diffstat (limited to 'src')
-rw-r--r--src/thread/x86_64/clone.s12
1 files changed, 6 insertions, 6 deletions
diff --git a/src/thread/x86_64/clone.s b/src/thread/x86_64/clone.s
index 51410051..3cefd933 100644
--- a/src/thread/x86_64/clone.s
+++ b/src/thread/x86_64/clone.s
@@ -4,6 +4,7 @@
 .type   __uniclone,%function
 /* rdi = child_stack, rsi = start, rdx = pthread_struct */
 __uniclone:
+        subq    $8,%rsp         /* pad parent stack to prevent branch later */
         subq    $16,%rdi        /* grow child_stack */
         mov     %rsi,8(%rdi)    /* push start onto child_stack as return ptr */
         mov     %rdx,0(%rdi)    /* push pthread_struct onto child_stack */
@@ -13,10 +14,9 @@ __uniclone:
         movl    $56,%eax        /* clone syscall number */
         movl    $0x7d0f00,%edi  /* rdi = flags */
         mov     %r10,%rdx       /* rdx = parent_id */
-	syscall                 /* clone(flags, child_stack, parent_id,
-	                         *       child_id, tls) */
-	test	%rax,%rax
-	jnz	1f              /* if we're in the parent -> goto 1f */
-        pop     %rdi            /* restore pthread_struct from child stack */
-1:      ret
+        syscall                 /* clone(flags, child_stack, parent_id,
+                                 *       child_id, tls) */
+        pop     %rdi            /* child stack: restore pthread_struct
+                                 * parent stack: undo rsp displacement */
+        ret
 .size __uniclone,.-__uniclone